import { BaseTrigger, Operator } from '../type';
export declare const HttpTriggerType = "HTTP";
export declare enum HttpMetadata {
    METHOD = "Http_Method",
    QUERY = "Http_Query",
    PARAMS = "Http_Params",
    HEADERS = "Http_Headers",
    RESPONSE = "Http_Response"
}
export declare enum ResponseMetaType {
    CODE = "Http_Response_Code",
    HEADER = "Http_Response_Header",
    CONTENT_TYPE = "Http_Response_ContentType",
    REDIRECT = "Http_Response_Redirect"
}
export declare enum HttpMethod {
    GET = "GET",
    POST = "POST",
    PUT = "PUT",
    DELETE = "DELETE",
    PATCH = "PATCH",
    HEAD = "HEAD",
    OPTIONS = "OPTIONS",
    ALL = "ALL"
}
export interface HttpTrigger extends BaseTrigger {
    type: typeof HttpTriggerType;
    method: HttpMethod;
    path?: string;
}
export declare type HttpInputMetadata = {
    query?: Record<string, string>;
    headers?: Record<string, string>;
    params?: Record<string, string>;
};
export declare const All: (path?: string) => Operator<void>;
export declare const Get: (path?: string) => Operator<void>;
export declare const Post: (path?: string) => Operator<void>;
export declare const Delete: (path?: string) => Operator<void>;
export declare const Put: (path?: string) => Operator<void>;
export declare const Patch: (path?: string) => Operator<void>;
export declare const Head: (path?: string) => Operator<void>;
export declare const Options: (path?: string) => Operator<void>;
export declare function Query<T extends Record<string, string>>(): Operator<{
    query: T;
}>;
export declare function Params<T extends Record<string, string>>(): Operator<{
    params: T;
}>;
export declare function Headers<T extends Record<string, string>>(): Operator<{
    headers: T;
}>;
export declare type ResponseMetaData = {
    type: ResponseMetaType;
    code?: number;
    header?: {
        key: string;
        value: string;
    };
    url?: string;
    contentType?: string;
};
export declare function HttpCode(code: number): Operator<void>;
export declare function SetHeader(key: string, value: string): Operator<void>;
export declare function Redirect(url: string, code?: number): Operator<void>;
export declare function ContentType(contentType: string): Operator<void>;
