'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var asynchronousLocalStorage = require('@midwayjs/asynchronous-local-storage');
var upath = require('upath');
var urlJoin = require('proper-url-join');
var some = require('lodash/some');
require('reflect-metadata');
var parseArgs = require('fn-args');
var debug$2 = require('debug');
var isClass = require('is-class');
var util = require('util');
var pickBy = require('lodash/pickBy');
var compose = require('koa-compose');

function _interopDefault (e) { return e && e.__esModule ? e : { 'default': e }; }

var urlJoin__default = /*#__PURE__*/_interopDefault(urlJoin);
var some__default = /*#__PURE__*/_interopDefault(some);
var parseArgs__default = /*#__PURE__*/_interopDefault(parseArgs);
var debug__default = /*#__PURE__*/_interopDefault(debug$2);
var isClass__default = /*#__PURE__*/_interopDefault(isClass);
var pickBy__default = /*#__PURE__*/_interopDefault(pickBy);
var compose__default = /*#__PURE__*/_interopDefault(compose);

const AsyncLocalStorageRuntime = {
  name: 'AsyncLocalStorageRuntime',
  getValue(key) {
    return asynchronousLocalStorage.als.get(key);
  },
  run(ctx, callback) {
    return new Promise((resolve, reject) => {
      asynchronousLocalStorage.als.runWith(() => callback().then(resolve).catch(reject), ctx);
    });
  } };

const ContextManager = {
  getValue(key) {
    return AsyncLocalStorageRuntime.getValue(key);
  },
  run(ctx, callback) {
    return AsyncLocalStorageRuntime.run(ctx, callback);
  } };


function useContext() {
  return ContextManager.getValue('ctx');
}

class AbstractRouter {














  isSourceFile(file, source) {
    return this.isJavaScriptFile(file) && this.isPathInside(file, source);
  }

  isPathInside(child, parent) {
    const relation = upath.relative(parent, child);
    return Boolean(
    relation &&
    relation !== '..' &&
    !relation.startsWith(`..${upath.sep}`) &&
    relation !== upath.resolve(child));

  }

  isJavaScriptFile(file) {
    const extensions = ['.ts', '.js', '.mjs'];
    if (!extensions.includes(upath.extname(file))) {
      return false;
    }

    const testExt = ['.test.ts', '.test.js', '.test.mjs'];

    if (testExt.some((ext) => file.endsWith(ext))) {
      return false;
    }

    return !file.includes('_client');
  }}

exports.OperatorType = void 0;(function (OperatorType) {OperatorType["Trigger"] = "Trigger";OperatorType["Middleware"] = "Middleware";})(exports.OperatorType || (exports.OperatorType = {}));

const EXPORT_DEFAULT_FUNCTION_ALIAS = '$default';
const USE_INPUT_METADATA = 'USE_INPUT_METADATA';
const API_BASE_PATH = '/api';

const HttpTriggerType = 'HTTP';

exports.HttpMetadata = void 0;(function (HttpMetadata) {HttpMetadata["METHOD"] = "Http_Method";HttpMetadata["QUERY"] = "Http_Query";HttpMetadata["PARAMS"] = "Http_Params";HttpMetadata["HEADERS"] = "Http_Headers";HttpMetadata["RESPONSE"] = "Http_Response";})(exports.HttpMetadata || (exports.HttpMetadata = {}));







exports.ResponseMetaType = void 0;(function (ResponseMetaType) {ResponseMetaType["CODE"] = "Http_Response_Code";ResponseMetaType["HEADER"] = "Http_Response_Header";ResponseMetaType["CONTENT_TYPE"] = "Http_Response_ContentType";ResponseMetaType["REDIRECT"] = "Http_Response_Redirect";})(exports.ResponseMetaType || (exports.ResponseMetaType = {}));






exports.HttpMethod = void 0;(function (HttpMethod) {HttpMethod["GET"] = "GET";HttpMethod["POST"] = "POST";HttpMethod["PUT"] = "PUT";HttpMethod["DELETE"] = "DELETE";HttpMethod["PATCH"] = "PATCH";HttpMethod["HEAD"] = "HEAD";HttpMethod["OPTIONS"] = "OPTIONS";HttpMethod["ALL"] = "ALL";})(exports.HttpMethod || (exports.HttpMethod = {}));






















function createHttpMethodOperator(method) {
  return (path) => {
    return {
      name: method,
      metadata({ setMetadata }) {
        setMetadata(exports.OperatorType.Trigger, {
          type: HttpTriggerType,
          method,
          path,
          requestClient: {
            fetcher: 'http',
            client: '@midwayjs/rpc' } });


      } };

  };
}

// HTTP Method
const All = createHttpMethodOperator(exports.HttpMethod.ALL);
const Get = createHttpMethodOperator(exports.HttpMethod.GET);
const Post = createHttpMethodOperator(exports.HttpMethod.POST);
const Delete = createHttpMethodOperator(exports.HttpMethod.DELETE);
const Put = createHttpMethodOperator(exports.HttpMethod.PUT);
const Patch = createHttpMethodOperator(exports.HttpMethod.PATCH);
const Head = createHttpMethodOperator(exports.HttpMethod.HEAD);
const Options = createHttpMethodOperator(exports.HttpMethod.OPTIONS);

// HTTP Helper
function Query()

{
  return {
    name: exports.HttpMetadata.QUERY,
    input: true,
    metadata({ setMetadata }) {
      setMetadata(exports.HttpMetadata.QUERY, true);
    } };

}

function Params()

{
  return {
    name: exports.HttpMetadata.PARAMS,
    input: true,
    metadata({ setMetadata }) {
      setMetadata(exports.HttpMetadata.PARAMS, true);
    } };

}

function Headers()

{
  return {
    name: exports.HttpMetadata.HEADERS,
    input: true,
    metadata({ setMetadata }) {
      setMetadata(exports.HttpMetadata.HEADERS, true);
    } };

}












function HttpCode(code) {
  return {
    name: 'HttpCode',
    metadata(helper) {
      setResponseMetaData(helper, exports.ResponseMetaType.CODE, { code });
    } };

}

function SetHeader(key, value) {
  return {
    name: 'SetHeader',
    metadata(helper) {
      setResponseMetaData(helper, exports.ResponseMetaType.HEADER, {
        header: {
          key,
          value } });


    } };

}

function Redirect(url, code) {
  return {
    name: 'Redirect',
    metadata(helper) {
      setResponseMetaData(helper, exports.ResponseMetaType.REDIRECT, {
        url,
        code });

    } };

}

function ContentType(contentType) {
  return {
    name: 'ContentType',
    metadata(helper) {
      setResponseMetaData(helper, exports.ResponseMetaType.CONTENT_TYPE, {
        contentType });

    } };

}

function setResponseMetaData(
helper,
type,
value)
{
  const responseMetaData =
  helper.getMetadata(exports.HttpMetadata.RESPONSE) || [];

  helper.setMetadata(exports.HttpMetadata.RESPONSE, [
  ...responseMetaData,
  {
    type,
    ...value }]);


}

function isFunction(arg) {
  return typeof arg === 'function' && !isClass__default["default"](arg);
}

function isHooksMiddleware(fn) {
  return isFunction(fn) && parseArgs__default["default"](fn).length === 1;
}

function extractMetadata(target) {
  const metadata = {};
  const metaKeys = Reflect.getMetadataKeys(target);
  for (const key of metaKeys) {
    metadata[key] = Reflect.getMetadata(key, target);
  }
  return metadata;
}

function createDebug(namespace) {
  return debug__default["default"](namespace);
}

class ERR_INVALID_ARG_VALUE extends Error {
  constructor(actual, name, reason = 'is invalid') {
    const message = `[ERR_INVALID_ARG_VALUE]: The argument '${name}' ${reason} ${getTypeMessage(
    actual)
    }`;
    super(message);
  }}


class ERR_INVALID_ARG_TYPE extends Error {
  constructor(name, expected, actual) {
    const message = `[ERR_INVALID_ARG_TYPE]: The '${name}' argument must be of type ${expected}${getTypeMessage(
    actual)
    }`;
    super(message);
  }}


// fork from https://github.com/nodejs/node/blob/master/lib/internal/errors.js
function getTypeMessage(actual) {
  let msg = '';
  if (actual == null) {
    msg += `. Received ${actual}`;
  } else if (typeof actual === 'function' && actual.name) {
    msg += `. Received function ${actual.name}`;
  } else if (typeof actual === 'object') {
    if (actual.constructor && actual.constructor.name) {
      msg += `. Received an instance of ${actual.constructor.name}`;
    } else {
      const inspected = util.inspect(actual, { depth: -1 });
      msg += `. Received ${inspected}`;
    }
  } else {
    let inspected = util.inspect(actual, { colors: false });
    if (inspected.length > 25) inspected = `${inspected.slice(0, 25)}...`;
    msg += `. Received type ${typeof actual} (${inspected})`;
  }
  return msg;
}

function validateArray(value, name) {
  if (!Array.isArray(value)) {
    throw new ERR_INVALID_ARG_TYPE(name, 'Array', value);
  }
}

function validateString(value, name) {
  if (typeof value !== 'string')
  throw new ERR_INVALID_ARG_TYPE(name, 'string', value);
}

function validateFunction(value, name) {
  if (typeof value !== 'function') {
    throw new ERR_INVALID_ARG_TYPE(name, 'Function', value);
  }
}

function validateOneOf(value, name, oneOf) {
  if (!oneOf.includes(value)) {
    const allowed = oneOf.
    map((v) => typeof v === 'string' ? `'${v}'` : String(v)).
    join(', ');
    const reason = 'must be one of: ' + allowed;
    throw new ERR_INVALID_ARG_VALUE(name, value, reason);
  }
}

const debug$1 = createDebug('hooks-core: adapter');






exports.framework = null;

function useFrameworkAdapter(adapter) {
  validateFunction(
  adapter.handleResponseMetaData,
  'adapter.handleResponseMetaData');

  debug$1('use framework adapter: %s', adapter.name);
  exports.framework = adapter;
}

function Api(



...args)





{
  const handler = args.pop();
  validateFunction(handler, 'ApiHandler');

  const operators = args;

  const metadataHelper = {
    getMetadata(key) {
      return Reflect.getMetadata(key, runner);
    },
    setMetadata(key, value) {
      return Reflect.defineMetadata(key, value, runner);
    } };


  for (const operator of operators) {
    if (operator.metadata) {
      validateFunction(operator.metadata, 'operator.metadata');
      operator.metadata(metadataHelper);
    }
  }

  const useInputMetadata = operators.some((operator) => operator.input);
  const executors = operators.
  filter((operator) => typeof operator.execute === 'function').
  map((operator) => operator.execute);

  async function runner(...args) {
    const stack = [...executors];

    const executeHelper = {
      result: undefined,
      getInputArguments: () => args };


    stack.push(async (helper, next) => {
      helper.result = await handler(...args);
      return next();
    });

    await compose__default["default"](stack)(executeHelper);

    // handle HttpCode/Redirect/etc.
    const responseMetadata = Reflect.getMetadata(exports.HttpMetadata.RESPONSE, runner);
    if (Array.isArray(responseMetadata)) {
      await exports.framework.handleResponseMetaData(responseMetadata);
    }
    return executeHelper.result;
  }

  Reflect.defineMetadata(USE_INPUT_METADATA, useInputMetadata, runner);
  return runner;
}

function Middleware(...middlewares) {
  let middleware = middlewares;
  if (Array.isArray(middlewares[0])) {
    middleware = middlewares[0];
  }

  return {
    name: 'Middleware',
    metadata({ setMetadata }) {
      setMetadata(exports.OperatorType.Middleware, middleware);
    } };

}

let validator = (schemas, input) => {
  throw new Error('validator is not set');
};

function setValidator(newValidator) {
  validateFunction(newValidator, 'newValidator');
  validator = newValidator;
}

function Validate(...schemas) {
  return {
    name: 'Validate',
    async execute({ getInputArguments }, next) {
      validateFunction(validator, 'validator');
      const inputs = getInputArguments();
      await validator(schemas, inputs);
      return next();
    } };

}

function getApiTrigger(
target)
{
  return Reflect.getOwnMetadata(exports.OperatorType.Trigger, target);
}

class ApiRouter extends AbstractRouter {
  constructor(config = {}) {var _config$basePath;
    super();this.config = config;
    (_config$basePath = config.basePath) !== null && _config$basePath !== void 0 ? _config$basePath : config.basePath = API_BASE_PATH;
  }

  isApiFile(options) {
    return this.hasExportApiRoutes(options.mod);
  }

  hasExportApiRoutes(mod) {
    return some__default["default"](
    mod,
    (exportVariable) =>
    isFunction(exportVariable) &&
    !!Reflect.getMetadata(exports.OperatorType.Trigger, exportVariable));

  }

  getFunctionId(
  file,
  functionName,
  exportDefault)
  {
    return exportDefault ?
    upath.removeExt(upath.basename(file), upath.extname(file)) :
    functionName;
  }

  functionToHttpPath(
  file,
  functionName,
  exportDefault)
  {
    return urlJoin__default["default"](
    this.config.basePath,
    this.getFunctionId(file, functionName, exportDefault),
    { trailingSlash: false });

  }}

const debug = createDebug('hooks-core: loader');




















function parseApiModule(
mod,
file,
router)
{
  const apis = [];
  const funcs = pickBy__default["default"](mod, isFunction);
  for (let [name, fn] of Object.entries(funcs)) {var _mod$config;
    const exportDefault = name === 'default';
    const functionName = exportDefault ? EXPORT_DEFAULT_FUNCTION_ALIAS : name;
    const functionId = router.getFunctionId(file, functionName, exportDefault);

    // default is http trigger
    let trigger = Reflect.getMetadata(exports.OperatorType.Trigger, fn);

    // File Router
    if (!trigger) {
      // default is http
      const HttpMethod = parseArgs__default["default"](fn).length === 0 ? Get : Post;
      // wrap pure function
      fn = Api(HttpMethod(), fn);
      mod[name] = fn;
      // get trigger
      trigger = Reflect.getMetadata(exports.OperatorType.Trigger, fn);
    }

    // Http Path
    if (trigger.type === HttpTriggerType) {var _trigger, _trigger$path;
      (_trigger$path = (_trigger = trigger).path) !== null && _trigger$path !== void 0 ? _trigger$path : _trigger.path = router.functionToHttpPath(
      file,
      functionName,
      exportDefault);

      debug('trigger: %o', trigger);
    }

    // Middleware
    const fnMiddleware = Reflect.getMetadata(exports.OperatorType.Middleware, fn) || [];
    const fileMiddleware = (mod === null || mod === void 0 ? void 0 : (_mod$config = mod.config) === null || _mod$config === void 0 ? void 0 : _mod$config.middleware) || [];
    const middleware = [...fnMiddleware, ...fileMiddleware];

    apis.push({
      fn,
      file,
      functionName,
      functionId,
      trigger,
      middleware,
      useInputMetadata: Reflect.getMetadata(USE_INPUT_METADATA, fn) });

  }

  return apis;
}

Object.defineProperty(exports, 'urlJoin', {
  enumerable: true,
  get: function () { return urlJoin__default["default"]; }
});
exports.API_BASE_PATH = API_BASE_PATH;
exports.AbstractRouter = AbstractRouter;
exports.All = All;
exports.Api = Api;
exports.ApiRouter = ApiRouter;
exports.AsyncLocalStorageRuntime = AsyncLocalStorageRuntime;
exports.ContentType = ContentType;
exports.ContextManager = ContextManager;
exports.Delete = Delete;
exports.ERR_INVALID_ARG_TYPE = ERR_INVALID_ARG_TYPE;
exports.ERR_INVALID_ARG_VALUE = ERR_INVALID_ARG_VALUE;
exports.EXPORT_DEFAULT_FUNCTION_ALIAS = EXPORT_DEFAULT_FUNCTION_ALIAS;
exports.Get = Get;
exports.Head = Head;
exports.Headers = Headers;
exports.HttpCode = HttpCode;
exports.HttpTriggerType = HttpTriggerType;
exports.Middleware = Middleware;
exports.Options = Options;
exports.Params = Params;
exports.Patch = Patch;
exports.Post = Post;
exports.Put = Put;
exports.Query = Query;
exports.Redirect = Redirect;
exports.SetHeader = SetHeader;
exports.USE_INPUT_METADATA = USE_INPUT_METADATA;
exports.Validate = Validate;
exports.createDebug = createDebug;
exports.extractMetadata = extractMetadata;
exports.getApiTrigger = getApiTrigger;
exports.isFunction = isFunction;
exports.isHooksMiddleware = isHooksMiddleware;
exports.parseApiModule = parseApiModule;
exports.setValidator = setValidator;
exports.useContext = useContext;
exports.useFrameworkAdapter = useFrameworkAdapter;
exports.validateArray = validateArray;
exports.validateFunction = validateFunction;
exports.validateOneOf = validateOneOf;
exports.validateString = validateString;
